@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -126,10 +126,18 @@ static PrefParam param[] = {
 	{"enable_rules_hint", "TRUE", &prefs_common.enable_rules_hint, P_BOOL},
 	{"bold_unread", "TRUE", &prefs_common.bold_unread, P_BOOL},
 
-	{"toolbar_style", "3", &prefs_common.toolbar_style, P_ENUM},
+	{"persist_qsearch_filter", "TRUE", &prefs_common.persist_qsearch_filter,
+	 P_BOOL},
+
+	{"toolbar_style", "4", &prefs_common.toolbar_style, P_ENUM},
 	{"show_searchbar", "TRUE", &prefs_common.show_searchbar, P_BOOL},
 	{"show_statusbar", "TRUE", &prefs_common.show_statusbar, P_BOOL},
 
+	{"main_toolbar_setting", NULL, &prefs_common.main_toolbar_setting,
+	 P_STRING},
+	{"compose_toolbar_setting", NULL,
+	 &prefs_common.compose_toolbar_setting, P_STRING},
+
 	{"summary_col_show_mark", "TRUE",
 	 &prefs_common.summary_col_visible[S_COL_MARK], P_BOOL},
 	{"summary_col_show_unread", "TRUE",
@@ -339,6 +347,12 @@ static PrefParam param[] = {
 
 	{"show_trayicon", "TRUE", &prefs_common.show_trayicon, P_BOOL},
 	{"minimize_to_tray", "FALSE", &prefs_common.minimize_to_tray, P_BOOL},
+#ifdef G_OS_WIN32
+	{"toggle_window_on_trayicon_click", "FALSE",
+#else
+	{"toggle_window_on_trayicon_click", "TRUE",
+#endif
+	 &prefs_common.toggle_window_on_trayicon_click, P_BOOL},
 
 	/* Other */
 	{"receive_dialog_mode", "1", &prefs_common.recv_dialog_mode, P_ENUM},
@@ -415,15 +429,6 @@ void prefs_common_read_config(void)
 
 	prefs_read_config(param, "Common", path, NULL);
 
-#ifdef G_OS_WIN32
-	if (!is_file_exist(path)) {
-		if (conv_is_ja_locale()) {
-			g_free(prefs_common.textfont);
-			prefs_common.textfont = g_strdup("MS Gothic 12");
-		}
-	}
-#endif
-
 	g_free(path);
 
 	prefs_common.online_mode = TRUE;
