@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -114,10 +114,15 @@ struct _PrefsCommon
 	gboolean enable_rules_hint;
 	gboolean bold_unread;
 
+	gboolean persist_qsearch_filter;
+
 	ToolbarStyle toolbar_style;
 	gboolean show_searchbar;
 	gboolean show_statusbar;
 
+	gchar *main_toolbar_setting;
+	gchar *compose_toolbar_setting;
+
 	/* Summary columns visibility, position and size */
 	gboolean summary_col_visible[N_SUMMARY_COLS];
 	gint summary_col_pos[N_SUMMARY_COLS];
@@ -240,6 +245,7 @@ struct _PrefsCommon
 	gboolean comply_gnome_hig;
 	gboolean show_trayicon;
 	gboolean minimize_to_tray;
+	gboolean toggle_window_on_trayicon_click;
 
 	/* Other */
 	RecvDialogMode recv_dialog_mode;
