@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -41,6 +41,10 @@ static gint session_close		(Session	*session);
 
 static gboolean session_timeout_cb	(gpointer	 data);
 
+#ifdef G_OS_WIN32
+static gboolean session_ping_cb		(gpointer	 data);
+#endif
+
 static gboolean session_recv_msg_idle_cb	(gpointer	 data);
 static gboolean session_recv_data_idle_cb	(gpointer	 data);
 
@@ -107,6 +111,9 @@ void session_init(Session *session)
 	session->timeout_tag = 0;
 	session->timeout_interval = 0;
 
+	session->idle_tag = 0;
+	session->ping_tag = 0;
+
 	session->data = NULL;
 }
 
@@ -167,15 +174,19 @@ static gint session_connect_cb(SockInfo *sock, gpointer data)
 	}
 #endif
 
-	sock_set_nonblocking_mode(sock, session->nonblocking);
-
 	debug_print("session (%p): connected\n", session);
 
+	sock_set_nonblocking_mode(sock, session->nonblocking);
+
 	session->state = SESSION_RECV;
 	session->io_tag = sock_add_watch(session->sock, G_IO_IN,
 					 session_read_msg_cb,
 					 session);
 
+#ifdef G_OS_WIN32
+	session->ping_tag = g_timeout_add(1000, session_ping_cb, session);
+#endif
+
 	return 0;
 }
 
@@ -247,6 +258,39 @@ static gboolean session_timeout_cb(gpointer data)
 	return FALSE;
 }
 
+#ifdef G_OS_WIN32
+/* hack for state machine freeze problem in GLib >= 2.8.x */
+static gboolean session_ping_cb(gpointer data)
+{
+	Session *session = SESSION(data);
+	SockInfo *sock = session->sock;
+
+	if (session->io_tag > 0 && sock && sock->callback) {
+		GTimeVal tv_cur, tv_result;
+
+		g_get_current_time(&tv_cur);
+		tv_result.tv_sec = tv_cur.tv_sec - session->tv_prev.tv_sec;
+		tv_result.tv_usec = tv_cur.tv_usec - session->tv_prev.tv_usec;
+		if (tv_result.tv_usec < 0) {
+			tv_result.tv_sec--;
+			tv_result.tv_usec += G_USEC_PER_SEC;
+		}
+		if (tv_result.tv_sec * G_USEC_PER_SEC + tv_result.tv_usec >
+		    G_USEC_PER_SEC) {
+			SockFlags save_flags;
+
+			debug_print("state machine freeze for 1 second detected, forcing dispatch.\n");
+			save_flags = sock->flags;
+			SOCK_UNSET_FLAGS(sock->flags, SOCK_CHECK_IO);
+			sock->callback(sock, sock->condition, sock->data);
+			sock->flags = save_flags;
+		}
+	}
+
+	return TRUE;
+}
+#endif
+
 void session_set_recv_message_notify(Session *session,
 				     RecvMsgNotify notify_func, gpointer data)
 {
@@ -300,6 +344,18 @@ static gint session_close(Session *session)
 
 	session_set_timeout(session, 0);
 
+	if (session->idle_tag > 0) {
+		g_source_remove(session->idle_tag);
+		session->idle_tag = 0;
+	}
+
+#ifdef G_OS_WIN32
+	if (session->ping_tag > 0) {
+		g_source_remove(session->ping_tag);
+		session->ping_tag = 0;
+	}
+#endif
+
 	if (session->io_tag > 0) {
 		g_source_remove(session->io_tag);
 		session->io_tag = 0;
@@ -322,8 +378,7 @@ gint session_start_tls(Session *session)
 
 	nb_mode = sock_is_nonblocking_mode(session->sock);
 
-	if (nb_mode)
-		sock_set_nonblocking_mode(session->sock, FALSE);
+	sock_set_nonblocking_mode(session->sock, FALSE);
 
 	if (!ssl_init_socket_with_method(session->sock, SSL_METHOD_TLSv1)) {
 		g_warning("can't start TLS session.\n");
@@ -332,8 +387,7 @@ gint session_start_tls(Session *session)
 		return -1;
 	}
 
-	if (nb_mode)
-		sock_set_nonblocking_mode(session->sock, session->nonblocking);
+	sock_set_nonblocking_mode(session->sock, session->nonblocking);
 
 	return 0;
 }
@@ -370,7 +424,8 @@ gint session_recv_msg(Session *session)
 	session->state = SESSION_RECV;
 
 	if (session->read_buf_len > 0)
-		g_idle_add(session_recv_msg_idle_cb, session);
+		session->idle_tag = g_idle_add(session_recv_msg_idle_cb,
+					       session);
 	else
 		session->io_tag = sock_add_watch(session->sock, G_IO_IN,
 						 session_read_msg_cb, session);
@@ -383,6 +438,12 @@ static gboolean session_recv_msg_idle_cb(gpointer data)
 	Session *session = SESSION(data);
 	gboolean ret;
 
+#if GLIB_CHECK_VERSION(2, 12, 0)
+	if (g_source_is_destroyed(g_main_current_source()))
+		return FALSE;
+#endif
+
+	session->idle_tag = 0;
 	ret = session_read_msg_cb(session->sock, G_IO_IN, session);
 
 	if (ret == TRUE)
@@ -430,7 +491,8 @@ gint session_recv_data(Session *session, guint size, const gchar *terminator)
 	g_get_current_time(&session->tv_prev);
 
 	if (session->read_buf_len > 0)
-		g_idle_add(session_recv_data_idle_cb, session);
+		session->idle_tag = g_idle_add(session_recv_data_idle_cb,
+					       session);
 	else
 		session->io_tag = sock_add_watch(session->sock, G_IO_IN,
 						 session_read_data_cb, session);
@@ -443,6 +505,12 @@ static gboolean session_recv_data_idle_cb(gpointer data)
 	Session *session = SESSION(data);
 	gboolean ret;
 
+#if GLIB_CHECK_VERSION(2, 12, 0)
+	if (g_source_is_destroyed(g_main_current_source()))
+		return FALSE;
+#endif
+
+	session->idle_tag = 0;
 	ret = session_read_data_cb(session->sock, G_IO_IN, session);
 
 	if (ret == TRUE)
@@ -471,7 +539,8 @@ gint session_recv_data_as_file(Session *session, guint size,
 	}
 
 	if (session->read_buf_len > 0)
-		g_idle_add(session_recv_data_as_file_idle_cb, session);
+		session->idle_tag =
+			g_idle_add(session_recv_data_as_file_idle_cb, session);
 	else
 		session->io_tag = sock_add_watch(session->sock, G_IO_IN,
 						 session_read_data_as_file_cb,
@@ -485,6 +554,12 @@ static gboolean session_recv_data_as_file_idle_cb(gpointer data)
 	Session *session = SESSION(data);
 	gboolean ret;
 
+#if GLIB_CHECK_VERSION(2, 12, 0)
+	if (g_source_is_destroyed(g_main_current_source()))
+		return FALSE;
+#endif
+
+	session->idle_tag = 0;
 	ret = session_read_data_as_file_cb(session->sock, G_IO_IN, session);
 
 	if (ret == TRUE)
@@ -507,13 +582,9 @@ static gboolean session_read_msg_cb(SockInfo *source, GIOCondition condition,
 
 	g_return_val_if_fail(condition == G_IO_IN, FALSE);
 
-	session_set_timeout(session, session->timeout_interval);
-
 	if (session->read_buf_len == 0) {
 		gint read_len;
 
-		if (!sock_has_read_data(session->sock))
-			return TRUE;
 		read_len = sock_read(session->sock, session->read_buf,
 				     SESSION_BUFFSIZE - 1);
 
@@ -528,7 +599,7 @@ static gboolean session_read_msg_cb(SockInfo *source, GIOCondition condition,
 			case EAGAIN:
 				return TRUE;
 			default:
-				g_warning("sock_read: %s\n", g_strerror(errno));
+				g_warning("%s: sock_read: %s\n", G_STRFUNC, g_strerror(errno));
 				session->state = SESSION_ERROR;
 				return FALSE;
 			}
@@ -537,6 +608,8 @@ static gboolean session_read_msg_cb(SockInfo *source, GIOCondition condition,
 		session->read_buf_len = read_len;
 	}
 
+	session_set_timeout(session, session->timeout_interval);
+
 	if ((newline = memchr(session->read_buf_p, '\n', session->read_buf_len))
 		!= NULL)
 		line_len = newline - session->read_buf_p + 1;
@@ -573,7 +646,9 @@ static gboolean session_read_msg_cb(SockInfo *source, GIOCondition condition,
 	g_string_truncate(session->read_msg_buf, 0);
 
 	ret = session->recv_msg(session, msg);
-	session->recv_msg_notify(session, msg, session->recv_msg_notify_data);
+	if (session->recv_msg_notify)
+		session->recv_msg_notify(session, msg,
+					 session->recv_msg_notify_data);
 
 	g_free(msg);
 
@@ -595,13 +670,9 @@ static gboolean session_read_data_cb(SockInfo *source, GIOCondition condition,
 
 	g_return_val_if_fail(condition == G_IO_IN, FALSE);
 
-	session_set_timeout(session, session->timeout_interval);
-
 	if (session->read_buf_len == 0) {
 		gint read_len;
 
-		if (!sock_has_read_data(session->sock))
-			return TRUE;
 		read_len = sock_read(session->sock, session->read_buf,
 				     SESSION_BUFFSIZE);
 
@@ -616,7 +687,7 @@ static gboolean session_read_data_cb(SockInfo *source, GIOCondition condition,
 			case EAGAIN:
 				return TRUE;
 			default:
-				g_warning("sock_read: %s\n", g_strerror(errno));
+				g_warning("%s: sock_read: %s\n", G_STRFUNC, g_strerror(errno));
 				session->state = SESSION_ERROR;
 				return FALSE;
 			}
@@ -625,6 +696,8 @@ static gboolean session_read_data_cb(SockInfo *source, GIOCondition condition,
 		session->read_buf_len = read_len;
 	}
 
+	session_set_timeout(session, session->timeout_interval);
+
 	data_buf = session->read_data_buf;
 	terminator_len = strlen(session->read_data_terminator);
 
@@ -659,9 +732,10 @@ static gboolean session_read_data_cb(SockInfo *source, GIOCondition condition,
 		if (tv_cur.tv_sec - session->tv_prev.tv_sec > 0 ||
 		    tv_cur.tv_usec - session->tv_prev.tv_usec >
 		    UI_REFRESH_INTERVAL) {
-			session->recv_data_progressive_notify
-				(session, data_buf->len, 0,
-				 session->recv_data_progressive_notify_data);
+			if (session->recv_data_progressive_notify)
+				session->recv_data_progressive_notify
+					(session, data_buf->len, 0,
+					 session->recv_data_progressive_notify_data);
 			g_get_current_time(&session->tv_prev);
 		}
 		return TRUE;
@@ -681,8 +755,9 @@ static gboolean session_read_data_cb(SockInfo *source, GIOCondition condition,
 
 	g_byte_array_set_size(data_buf, 0);
 
-	session->recv_data_notify(session, data_len,
-				  session->recv_data_notify_data);
+	if (session->recv_data_notify)
+		session->recv_data_notify(session, data_len,
+					  session->recv_data_notify_data);
 
 	if (ret < 0)
 		session->state = SESSION_ERROR;
@@ -710,11 +785,7 @@ static gboolean session_read_data_as_file_cb(SockInfo *source,
 
 	g_return_val_if_fail(condition == G_IO_IN, FALSE);
 
-	session_set_timeout(session, session->timeout_interval);
-
 	if (session->read_buf_len == 0) {
-		if (!sock_has_read_data(session->sock))
-			return TRUE;
 		read_len = sock_read(session->sock, session->read_buf_p,
 				     READ_BUF_LEFT());
 
@@ -729,7 +800,7 @@ static gboolean session_read_data_as_file_cb(SockInfo *source,
 			case EAGAIN:
 				return TRUE;
 			default:
-				g_warning("sock_read: %s\n", g_strerror(errno));
+				g_warning("%s: sock_read: %s\n", G_STRFUNC, g_strerror(errno));
 				session->state = SESSION_ERROR;
 				return FALSE;
 			}
@@ -738,6 +809,8 @@ static gboolean session_read_data_as_file_cb(SockInfo *source,
 		session->read_buf_len = read_len;
 	}
 
+	session_set_timeout(session, session->timeout_interval);
+
 	terminator_len = strlen(session->read_data_terminator);
 
 	if (session->read_buf_len == 0)
@@ -814,9 +887,10 @@ static gboolean session_read_data_as_file_cb(SockInfo *source,
 		if (tv_cur.tv_sec - session->tv_prev.tv_sec > 0 ||
 		    tv_cur.tv_usec - session->tv_prev.tv_usec >
 		    UI_REFRESH_INTERVAL) {
-			session->recv_data_progressive_notify
-				(session, session->read_data_pos, 0,
-				 session->recv_data_progressive_notify_data);
+			if (session->recv_data_progressive_notify)
+				session->recv_data_progressive_notify
+					(session, session->read_data_pos, 0,
+					 session->recv_data_progressive_notify_data);
 			g_get_current_time(&session->tv_prev);
 		}
 
@@ -848,6 +922,10 @@ static gboolean session_read_data_as_file_cb(SockInfo *source,
 	}
 	rewind(session->read_data_fp);
 
+	session->preread_len = 0;
+	session->read_buf_len = 0;
+	session->read_buf_p = session->read_buf;
+
 	/* callback */
 	ret = session->recv_data_as_file_finished
 		(session, session->read_data_fp, session->read_data_pos);
@@ -855,13 +933,11 @@ static gboolean session_read_data_as_file_cb(SockInfo *source,
 	fclose(session->read_data_fp);
 	session->read_data_fp = NULL;
 
-	session->recv_data_notify(session, session->read_data_pos,
-				  session->recv_data_notify_data);
+	if (session->recv_data_notify)
+		session->recv_data_notify(session, session->read_data_pos,
+					  session->recv_data_notify_data);
 
 	session->read_data_pos = 0;
-	session->preread_len = 0;
-	session->read_buf_len = 0;
-	session->read_buf_p = session->read_buf;
 
 	if (ret < 0)
 		session->state = SESSION_ERROR;
@@ -912,9 +988,11 @@ static gint session_write_buf(Session *session)
 	return 0;
 }
 
+#define WRITE_DATA_BUFFSIZE	8192
+
 static gint session_write_data(Session *session)
 {
-	gchar buf[SESSION_BUFFSIZE];
+	gchar buf[WRITE_DATA_BUFFSIZE];
 	gint write_len;
 	gint to_write_len;
 
@@ -923,7 +1001,7 @@ static gint session_write_data(Session *session)
 	g_return_val_if_fail(session->write_data_len > 0, -1);
 
 	to_write_len = session->write_data_len - session->write_data_pos;
-	to_write_len = MIN(to_write_len, SESSION_BUFFSIZE);
+	to_write_len = MIN(to_write_len, WRITE_DATA_BUFFSIZE);
 	if (fread(buf, to_write_len, 1, session->write_data_fp) < 1) {
 		g_warning("session_write_data: reading data from file failed\n");
 		session->state = SESSION_ERROR;
@@ -1021,10 +1099,12 @@ static gboolean session_write_data_cb(SockInfo *source,
 		    tv_cur.tv_usec - session->tv_prev.tv_usec >
 		    UI_REFRESH_INTERVAL) {
 			session_set_timeout(session, session->timeout_interval);
-			session->send_data_progressive_notify
-				(session,
-				 session->write_data_pos, write_data_len,
-				 session->send_data_progressive_notify_data);
+			if (session->send_data_progressive_notify)
+				session->send_data_progressive_notify
+					(session,
+					 session->write_data_pos,
+					 write_data_len,
+					 session->send_data_progressive_notify_data);
 			g_get_current_time(&session->tv_prev);
 		}
 		return TRUE;
@@ -1037,8 +1117,9 @@ static gboolean session_write_data_cb(SockInfo *source,
 
 	/* callback */
 	ret = session->send_data_finished(session, write_data_len);
-	session->send_data_notify(session, write_data_len,
-				  session->send_data_notify_data);
+	if (session->send_data_notify)
+		session->send_data_notify(session, write_data_len,
+					  session->send_data_notify_data);
 
 	return FALSE;
 }
