@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -247,7 +247,8 @@ gint fd_open_inet(gushort port)
 	}
 
 	val = 1;
-	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val)) < 0) {
+	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&val,
+		       sizeof(val)) < 0) {
 		perror("setsockopt");
 		fd_close(sock);
 		return -1;
@@ -380,8 +381,13 @@ static gint set_nonblocking_mode(gint fd, gboolean nonblock)
 	}
 
 	sock = sock_find_from_fd(fd);
-	if (sock)
-		sock->nonblock = nonblock;
+	if (sock) {
+		if (nonblock) {
+			SOCK_SET_FLAGS(sock->flags, SOCK_NONBLOCK);
+		} else {
+			SOCK_UNSET_FLAGS(sock->flags, SOCK_NONBLOCK);
+		}
+	}
 	debug_print("set nonblocking mode to %d\n", nonblock);
 
 	return 0;
@@ -410,8 +416,13 @@ gint sock_set_nonblocking_mode(SockInfo *sock, gboolean nonblock)
 	g_return_val_if_fail(sock != NULL, -1);
 
 	ret = set_nonblocking_mode(sock->sock, nonblock);
-	if (ret == 0)
-		sock->nonblock = nonblock;
+	if (ret == 0) {
+		if (nonblock) {
+			SOCK_SET_FLAGS(sock->flags, SOCK_NONBLOCK);
+		} else {
+			SOCK_UNSET_FLAGS(sock->flags, SOCK_NONBLOCK);
+		}
+	}
 
 	return ret;
 }
@@ -422,8 +433,9 @@ static gboolean is_nonblocking_mode(gint fd)
 	SockInfo *sock;
 
 	sock = sock_find_from_fd(fd);
-	if (sock)
-		return sock->nonblock;
+	if (sock) {
+		return SOCK_IS_NONBLOCK(sock->flags);
+	}
 
 	return FALSE;
 #else
@@ -444,7 +456,7 @@ gboolean sock_is_nonblocking_mode(SockInfo *sock)
 	g_return_val_if_fail(sock != NULL, FALSE);
 
 #ifdef G_OS_WIN32
-	return sock->nonblock;
+	return SOCK_IS_NONBLOCK(sock->flags);
 #else
 	return is_nonblocking_mode(sock->sock);
 #endif
@@ -560,8 +572,10 @@ static gint fd_check_io(gint fd, GIOCondition cond)
 {
 	struct timeval timeout;
 	fd_set fds;
+	SockInfo *sock;
 
-	if (is_nonblocking_mode(fd))
+	sock = sock_find_from_fd(fd);
+	if (sock && !SOCK_IS_CHECK_IO(sock->flags))
 		return 0;
 
 	timeout.tv_sec  = io_timeout;
@@ -676,6 +690,33 @@ struct hostent *my_gethostbyname(const gchar *hostname)
 	return hp;
 }
 
+static void sock_set_buffer_size(gint sock)
+{
+#ifdef G_OS_WIN32
+	gint val;
+	guint len = sizeof(val);
+
+#define SOCK_BUFFSIZE	32768
+
+	getsockopt(sock, SOL_SOCKET, SO_SNDBUF, (char *)&val, &len);
+	if (val < SOCK_BUFFSIZE) {
+		val = SOCK_BUFFSIZE;
+		setsockopt(sock, SOL_SOCKET, SO_SNDBUF, (char *)&val, len);
+	}
+	getsockopt(sock, SOL_SOCKET, SO_RCVBUF, (char *)&val, &len);
+	if (val < SOCK_BUFFSIZE) {
+		val = SOCK_BUFFSIZE;
+		setsockopt(sock, SOL_SOCKET, SO_RCVBUF, (char *)&val, len);
+	}
+	getsockopt(sock, SOL_SOCKET, SO_SNDBUF, (char *)&val, &len);
+	debug_print("SO_SNDBUF = %d\n", val);
+	getsockopt(sock, SOL_SOCKET, SO_RCVBUF, (char *)&val, &len);
+	debug_print("SO_RCVBUF = %d\n", val);
+
+#undef SOCK_BUFFSIZE
+#endif
+}
+
 #ifndef INET6
 static gint my_inet_aton(const gchar *hostname, struct in_addr *inp)
 {
@@ -771,6 +812,7 @@ static SockDesc sock_connect_by_getaddrinfo(const gchar *hostname, gushort port)
 		sock = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
 		if (!SOCKET_IS_VALID(sock))
 			continue;
+		sock_set_buffer_size(sock);
 
 		if (sock_connect_with_timeout
 			(sock, ai->ai_addr, ai->ai_addrlen, io_timeout) == 0)
@@ -808,6 +850,7 @@ SockInfo *sock_connect(const gchar *hostname, gushort port)
 #endif /* G_OS_WIN32 */
 		return NULL;
 	}
+	sock_set_buffer_size(sock);
 
 	if (sock_connect_by_hostname(sock, hostname, port) < 0) {
 		if (errno != 0) perror("connect");
@@ -822,7 +865,7 @@ SockInfo *sock_connect(const gchar *hostname, gushort port)
 	sockinfo->hostname = g_strdup(hostname);
 	sockinfo->port = port;
 	sockinfo->state = CONN_ESTABLISHED;
-	sockinfo->nonblock = FALSE;
+	sockinfo->flags = SOCK_CHECK_IO;
 
 	sock_list = g_list_prepend(sock_list, sockinfo);
 
@@ -882,7 +925,7 @@ static gboolean sock_connect_async_cb(GIOChannel *source,
 	sockinfo->hostname = g_strdup(conn_data->hostname);
 	sockinfo->port = conn_data->port;
 	sockinfo->state = CONN_ESTABLISHED;
-	sockinfo->nonblock = TRUE;
+	sockinfo->flags = SOCK_NONBLOCK;
 
 	sock_list = g_list_prepend(sock_list, sockinfo);
 
@@ -990,6 +1033,7 @@ static gint sock_connect_address_list_async(SockConnectData *conn_data)
 			continue;
 		}
 
+		sock_set_buffer_size(sock);
 		set_nonblocking_mode(sock, TRUE);
 
 		if (connect(sock, addr_data->addr, addr_data->addr_len) < 0) {
