@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -405,7 +405,9 @@ FolderView *folderview_create(void)
 	renderer = gtk_cell_renderer_text_new();
 	g_object_set(renderer, "xalign", 1.0, "ypad", 0, NULL);
 	column = gtk_tree_view_column_new_with_attributes
-		(_("New"), renderer, "text", COL_NEW, NULL);
+		(_("New"), renderer, "text", COL_NEW,
+		 "foreground-gdk", COL_FOREGROUND,
+		 "weight", COL_BOLD, NULL);
 	gtk_tree_view_column_set_alignment(column, 1.0);
 	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_fixed_width
@@ -419,7 +421,9 @@ FolderView *folderview_create(void)
 	renderer = gtk_cell_renderer_text_new();
 	g_object_set(renderer, "xalign", 1.0, "ypad", 0, NULL);
 	column = gtk_tree_view_column_new_with_attributes
-		(_("Unread"), renderer, "text", COL_UNREAD, NULL);
+		(_("Unread"), renderer, "text", COL_UNREAD,
+		 "foreground-gdk", COL_FOREGROUND,
+		 "weight", COL_BOLD, NULL);
 	gtk_tree_view_column_set_alignment(column, 1.0);
 	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_fixed_width
@@ -433,7 +437,9 @@ FolderView *folderview_create(void)
 	renderer = gtk_cell_renderer_text_new();
 	g_object_set(renderer, "xalign", 1.0, "ypad", 0, NULL);
 	column = gtk_tree_view_column_new_with_attributes
-		(_("#"), renderer, "text", COL_TOTAL, NULL);
+		(_("Total"), renderer, "text", COL_TOTAL,
+		 "foreground-gdk", COL_FOREGROUND,
+		 "weight", COL_BOLD, NULL);
 	gtk_tree_view_column_set_alignment(column, 1.0);
 	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_fixed_width
@@ -705,6 +711,7 @@ void folderview_select_next_unread(FolderView *folderview)
 {
 	GtkTreeModel *model = GTK_TREE_MODEL(folderview->store);
 	GtkTreeIter iter, next;
+	gboolean remember_last;
 
 	if (folderview->opened) {
 		GtkTreePath *path;
@@ -719,7 +726,10 @@ void folderview_select_next_unread(FolderView *folderview)
 			return;
 	}
 	if (folderview_find_next_unread(model, &next, &iter)) {
+		remember_last = prefs_common.remember_last_selected;
+		prefs_common.remember_last_selected = FALSE;
 		folderview_select_row(folderview, &next);
+		prefs_common.remember_last_selected = remember_last;
 		return;
 	}
 
@@ -727,8 +737,12 @@ void folderview_select_next_unread(FolderView *folderview)
 		return;
 
 	/* search again from the first row */
-	if (folderview_find_next_unread(model, &next, NULL))
+	if (folderview_find_next_unread(model, &next, NULL)) {
+		remember_last = prefs_common.remember_last_selected;
+		prefs_common.remember_last_selected = FALSE;
 		folderview_select_row(folderview, &next);
+		prefs_common.remember_last_selected = remember_last;
+	}
 }
 
 FolderItem *folderview_get_selected_item(FolderView *folderview)
@@ -896,7 +910,7 @@ static GtkWidget *label_window_create(const gchar *str)
 	gtk_container_add(GTK_CONTAINER(window), label);
 	gtk_widget_show(label);
 
-	gtk_widget_show_now(window);
+	gtk_widget_show(window);
 
 	return window;
 }
@@ -951,7 +965,7 @@ gint folderview_check_new(Folder *folder)
 	folderview = (FolderView *)folderview_list->data;
 	model = GTK_TREE_MODEL(folderview->store);
 
-	if (folder && !FOLDER_IS_LOCAL(folder)) {
+	if (folder && FOLDER_IS_REMOTE(folder)) {
 		if (!main_window_toggle_online_if_offline(folderview->mainwin))
 			return 0;
 	}
@@ -970,13 +984,14 @@ gint folderview_check_new(Folder *folder)
 		if (item->stype == F_VIRTUAL) continue;
 		if (item->no_select) continue;
 		if (folder && folder != item->folder) continue;
-		if (!folder && !FOLDER_IS_LOCAL(item->folder)) continue;
+		if (!folder && FOLDER_IS_REMOTE(item->folder)) continue;
 
 		prev_new = item->new;
 		prev_unread = item->unread;
 		folderview_scan_tree_func(item->folder, item, NULL);
 		if (folder_item_scan(item) < 0) {
-			if (folder && !FOLDER_IS_LOCAL(folder))
+			if (folder && FOLDER_IS_REMOTE(folder) &&
+			    REMOTE_FOLDER(folder)->session == NULL)
 				break;
 		}
 		folderview_update_row(folderview, &iter);
@@ -1659,7 +1674,7 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 		}
 		if (FOLDER_TYPE(folder) == F_IMAP ||
 		    FOLDER_TYPE(folder) == F_NEWS) {
-			if (item->parent != NULL && item->no_select == FALSE &&
+			if (item->no_select == FALSE &&
 			    item->stype != F_VIRTUAL)
 				download_msg = TRUE;
 		}
@@ -2070,6 +2085,7 @@ static void folderview_download_cb(FolderView *folderview, guint action,
 {
 	MainWindow *mainwin = folderview->mainwin;
 	FolderItem *item;
+	gint ret = 0;
 
 	item = folderview_get_selected_item(folderview);
 	if (!item)
@@ -2079,6 +2095,22 @@ static void folderview_download_cb(FolderView *folderview, guint action,
 
 	g_return_if_fail(item->folder != NULL);
 
+	if (item->parent == NULL) {
+		gchar *name, *msg;
+
+		name = trim_string(item->name, 32);
+		msg = g_strdup_printf(_("Download all messages under '%s' ?"),
+				      name);
+		g_free(name);
+		if (alertpanel(_("Download all messages"), msg,
+			       GTK_STOCK_YES, GTK_STOCK_NO, NULL)
+		    != G_ALERTDEFAULT) {
+			g_free(msg);
+			return;
+		}
+		g_free(msg);
+	}
+
 	if (!main_window_toggle_online_if_offline(folderview->mainwin))
 		return;
 
@@ -2089,13 +2121,40 @@ static void folderview_download_cb(FolderView *folderview, guint action,
 	main_window_progress_on(mainwin);
 	GTK_EVENTS_FLUSH();
 	folder_set_ui_func(item->folder, folderview_download_func, NULL);
-	if (folder_item_fetch_all_msg(item) < 0) {
+
+	if (item->parent == NULL) {
+		GtkTreeModel *model = GTK_TREE_MODEL(folderview->store);
+		GtkTreeIter iter;
+		gboolean valid;
+		FolderItem *cur_item;
+
+		valid = gtkut_tree_model_find_by_column_data
+			(model, &iter, NULL, COL_FOLDER_ITEM, item);
+		while ((valid = gtkut_tree_model_next(model, &iter)) == TRUE) {
+			cur_item = NULL;
+			gtk_tree_model_get(model, &iter, COL_FOLDER_ITEM,
+					   &cur_item, -1);
+			if (!cur_item || cur_item->folder != item->folder)
+				break;
+			if (!cur_item->no_select &&
+			    cur_item->stype != F_VIRTUAL &&
+			    cur_item->stype != F_TRASH) {
+				ret = folder_item_fetch_all_msg(cur_item);
+				if (ret < 0)
+					break;
+			}
+		}
+	} else
+		ret = folder_item_fetch_all_msg(item);
+
+	if (ret < 0) {
 		gchar *name;
 
 		name = trim_string(item->name, 32);
 		alertpanel_error(_("Error occurred while downloading messages in `%s'."), name);
 		g_free(name);
 	}
+
 	folder_set_ui_func(item->folder, NULL, NULL);
 	main_window_progress_off(mainwin);
 	gtk_widget_set_sensitive(folderview->treeview, TRUE);
